/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.pa165.soa.repositories;

import cz.muni.pa165.soa.Author;
import cz.muni.pa165.soa.Availability;
import cz.muni.pa165.soa.Book;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.PostConstruct;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

/**
 * This class represents resources of type Book that we will use in our application
 * Classes Book and Author are generated automatically from the XSD schema
 * in src/main/resources
 */
@Component
public class BookRepository {
    	private static final List<Book> books = new ArrayList<Book>();
        
        /**
         * Initialize the repository for books
         * we use @PostConstruct to initialize after dependency injection has been performed
         * see https://docs.oracle.com/javaee/7/api/javax/annotation/PostConstruct.html
         */
        @PostConstruct
	public void initData() {
            
            
            // ----- let's add several authors
            Author dougAdams = new Author();
            dougAdams.setName("Douglas");
            dougAdams.setSurname("Adams");
            
            Author williamGolding = new Author();
            williamGolding.setName("William");
            williamGolding.setSurname("Golding");
            
            Author gabrielMarquez = new Author();
            gabrielMarquez.setName("Gabriel Garcia");
            gabrielMarquez.setSurname("Marquez");
            
            Author rayBradbury = new Author();
            rayBradbury.setName("Ray");
            rayBradbury.setSurname("Bradbury");
            
            Author georgeOrwell = new Author();
            georgeOrwell.setName("George");
            georgeOrwell.setSurname("Orwell");
            
            Author dinoBuzzati = new Author();
            dinoBuzzati.setName("Dino");
            dinoBuzzati.setSurname("Buzzati");
           
            Author williamHodgson = new Author();
            williamHodgson.setName("William");
            williamHodgson.setSurname("Hodgson");
           
            // ----- let's add several books
            Book galaxyBook = new Book();
            galaxyBook.setAuthor(dougAdams);
            galaxyBook.setTitle("The Hitchhiker's Guide to the Galaxy");
            galaxyBook.setAvailability(Availability.ORDERED);
            galaxyBook.setPrice(55);
            galaxyBook.setIsbn("0345391802");
            books.add(galaxyBook);
            
            Book fliesBook = new Book();
            fliesBook.setAuthor(williamGolding);
            fliesBook.setTitle("Lord of the Flies");
            fliesBook.setAvailability(Availability.AVAILABLE);
            fliesBook.setPrice(25);
            fliesBook.setIsbn("0140283331");
            books.add(fliesBook);
            
            Book oneHundredBook = new Book();
            oneHundredBook.setAuthor(gabrielMarquez);
            oneHundredBook.setTitle("One Hundred Years of Solitude");
            oneHundredBook.setAvailability(Availability.AVAILABLE);
            oneHundredBook.setPrice(35);
            oneHundredBook.setIsbn("0060531045");
            books.add(oneHundredBook);
            
            Book fahrenheit451Book = new Book();
            fahrenheit451Book.setAuthor(rayBradbury);
            fahrenheit451Book.setTitle("Fahrenheit 451");
            fahrenheit451Book.setAvailability(Availability.SOLD_OUT);
            fahrenheit451Book.setPrice(22);
            fahrenheit451Book.setIsbn("0007491565");
            books.add(fahrenheit451Book);
            
            Book orwell1984Book = new Book();
            orwell1984Book.setAuthor(georgeOrwell);
            orwell1984Book.setTitle("1984");
            orwell1984Book.setAvailability(Availability.AVAILABLE);
            orwell1984Book.setPrice(23);
            orwell1984Book.setIsbn("0451524934");
            books.add(orwell1984Book);
            
            Book animalFarmBook = new Book();
            animalFarmBook.setAuthor(georgeOrwell);
            animalFarmBook.setTitle("Animal Farm");
            animalFarmBook.setAvailability(Availability.AVAILABLE);
            animalFarmBook.setPrice(25);
            animalFarmBook.setIsbn("0452284244");
            books.add(animalFarmBook);
            
            Book tartarSteppeBook = new Book();
            tartarSteppeBook.setAuthor(dinoBuzzati);
            tartarSteppeBook.setTitle("The Tartar Steppe");
            tartarSteppeBook.setAvailability(Availability.ORDERED);
            tartarSteppeBook.setPrice(20);
            tartarSteppeBook.setIsbn("1567923046");
            books.add(tartarSteppeBook);
            
            Book houseBordelandBook = new Book();
            houseBordelandBook.setAuthor(williamHodgson);
            houseBordelandBook.setTitle("The House on the Bordeland");
            houseBordelandBook.setAvailability(Availability.SOLD_OUT);
            houseBordelandBook.setPrice(15);
            houseBordelandBook.setIsbn("1426438281");
            books.add(houseBordelandBook);

	}
        
        /**
         * Get a book by specifying title
         * @param title
         * @return 
         */
        public Book getBookByTitle(String title) {
		Assert.notNull(title);

		Book result = null;

		for (Book book : books) {
			if (title.toLowerCase().equals(book.getTitle().toLowerCase())) {
				result = book;
			}
		}

		return result;
	}
        
       /**
        * Get the book(s) list by specifying the author
        * 
        * @param author
        * @return 
        */
        public Book getBookByAuthorNameAndSurname(Author author) {
		Assert.notNull(author);

		Book result = null;

		for (Book book : books) {
			if (author.getName().toLowerCase().equals(book.getAuthor().getName().toLowerCase() )
                                && (author.getSurname().toLowerCase().equals(book.getAuthor().getSurname().toLowerCase() )))  {
				result = book;
			}
		}

		return result;
	}

}
