package cz.muni.fi.pa165.soa.test;

import cz.muni.pa165.soa.app.Application;
import cz.muni.pa165.soa.app.WebServiceConfig;
import javax.xml.transform.Source;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.xml.transform.StringSource;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import org.springframework.ws.test.server.MockWebServiceClient;
import static org.springframework.ws.test.server.RequestCreators.*;
import static org.springframework.ws.test.server.ResponseMatchers.*;

/**
 * Integration test class for the BookEndpoint
 * we use MockWebServiceClient to build requests for the endpoint
 * and assert the return response payload
 * 
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {WebServiceConfig.class, Application.class})
public class BookEndpointIntegrationTest {

    @Autowired
    private ApplicationContext applicationContext;

    private MockWebServiceClient mockClient;

    @Before
    public void createClient() {
        mockClient = MockWebServiceClient.createClient(applicationContext);
    }

    /**
     * Test by passing a request based on a book's title expected return is a
     * payload with getBookResponse
     * This test is based on data from our service, it should be 
     * based on data created ad-hoc for the tests
     * @throws Exception
     */
    @Test
    public void bookEndpointGetBookRequest() throws Exception {

        Source requestPayload = new StringSource(
                "<getBookRequest xmlns='http://muni.cz/pa165/soa'>"
                + "<title>The Hitchhiker's Guide to the Galaxy</title>"
                + "</getBookRequest>");

        Source responsePayload = new StringSource(
                "<getBookResponse xmlns='http://muni.cz/pa165/soa'>"
                + "<book><title>The Hitchhiker's Guide to the Galaxy</title><isbn>0345391802"
                + "</isbn><price>55</price><author><name>Douglas</name><surname>Adams</surname></author>"
                + "<availability>ORDERED</availability>"
                + "</book></getBookResponse>");

        mockClient.sendRequest(withPayload(requestPayload)).
                andExpect(payload(responsePayload));
    }

    /**
     * Test by passing a request based on an author and getting back
     * payload with getBookResponse
     * This test is based on data from our service, it should be 
     * based on data created ad-hoc for the tests
     *
     * @throws Exception
     */
    @Test
    public void bookEndpointGetBookRequestByAuthor() throws Exception {

        Source requestPayload = new StringSource(
                "<getBookRequestByAuthor xmlns='http://muni.cz/pa165/soa'>"
                + "<author><name>William</name><surname>Golding</surname></author>"
                + "</getBookRequestByAuthor>");

        Source responsePayload = new StringSource(
                "<getBookResponse xmlns='http://muni.cz/pa165/soa'>"
                + "<book><title>Lord of the Flies</title><isbn>0140283331"
                + "</isbn><price>25</price><author><name>William</name><surname>Golding</surname></author>"
                + "<availability>AVAILABLE</availability>"
                + "</book></getBookResponse>");

        mockClient.sendRequest(withPayload(requestPayload)).
                andExpect(payload(responsePayload));
    }

}
